#include <Wire.h>
#include <SeeedOLED.h>
#include <DHT.h>
#include <SI114X.h>
#include <Arduino.h>

#define DHTPIN A1
#define DHTTYPE DHT11
SI114X SI1145 = SI114X();
DHT dht(DHTPIN, DHTTYPE);
int sensorPin = A0;
int sensorValue, vis, ir, uv;
int t, h;
void setup()
{
  Wire.begin();
  SeeedOled.init(); // initialze SEEED OLED display
  pinMode(4, OUTPUT);
  Serial.begin(9600);
}

void loop()
{
  vis = SI1145.ReadVisible(); //Read out Humidity value from Sensor
  ir = SI1145.ReadIR(); //Read out Temperature value from Sensor
  uv = SI1145.ReadUV(); //Read out Humidity value from Sensor
  sensorValue = analogRead(sensorPin);        //read out the moisture value from sensor
  sensorValue = map(sensorValue, 0, 500, 0, 100);   //map the moisture value to percentage
  h = dht.readHumidity(); //Read out Humidity value from Sensor
  t = dht.readTemperature(); //Read out Temperature value from Sensor

  Serial.print("V");
  Serial.println(vis);
  Serial.print("I");
  Serial.println(ir);
  Serial.print("U");
  Serial.println(uv);
  Serial.print("M");
  Serial.println(sensorValue);
  Serial.print("T");
  Serial.println(t);
  Serial.print("H");
  Serial.println(h);

  if (Serial.read() == 'W')                //if processing sends a W water the plant
    {
      digitalWrite(4, HIGH);              //the pump will turn on
      SeeedOled.setPageMode(); // Set addressing mode to Page Mode
      SeeedOled.clearDisplay();
      SeeedOled.setTextXY(2, 0);
      SeeedOled.putString("Watering in");
      SeeedOled.setTextXY(3, 0);
      SeeedOled.putString("progress ...");
      delay(9000);
      digitalWrite(4, LOW);
    }
    else
    {
      digitalWrite(4, LOW);
    }

  const unsigned long fiveMinutes = 5 * 60 * 1000UL;     // variable initialization for watering loop
  static unsigned long lastSampleTime = 0 - fiveMinutes; // initialize such that a reading is due the first time through loop()
  unsigned long now = millis();

  if (now - lastSampleTime >= fiveMinutes)        //check the moisture value every 5 minutes
  {
    lastSampleTime += fiveMinutes;
    if (sensorValue <= 65)                //if the moisture value falls below a certain threshhold
    {
      digitalWrite(4, HIGH);              //the pump will turn on
      SeeedOled.setPageMode(); // Set addressing mode to Page Mode
      SeeedOled.clearDisplay();
      SeeedOled.setTextXY(2, 0);
      SeeedOled.putString("Watering in");
      SeeedOled.setTextXY(3, 0);
      SeeedOled.putString("progress ...");
      delay(9000);
      digitalWrite(4, LOW);
    }
    else
    {
      digitalWrite(4, LOW);
    }
  }
  oled();
}

void oled() //controls all the outputs for the oled screen
{
  SeeedOled.setPageMode(); // Set addressing mode to Page Mode
  SeeedOled.clearDisplay();
  SeeedOled.setTextXY(2, 0);
  SeeedOled.putString("MoistureValue:");
  SeeedOled.setTextXY(4, 0);
  SeeedOled.putNumber(sensorValue); //Read out Moisture value from Sensor and Show it on the Screen
  SeeedOled.setTextXY(4, 2);
  SeeedOled.putString("%");
  delay(3000);
  SeeedOled.setPageMode(); // Set addressing mode to Page Mode
  SeeedOled.clearDisplay();
  SeeedOled.setTextXY(0, 0);
  SeeedOled.putString("Temperature:");
  SeeedOled.setTextXY(2, 0);
  SeeedOled.putNumber(t);
  SeeedOled.setTextXY(2, 2);
  SeeedOled.putString("°C");
  SeeedOled.setTextXY(4, 0);
  SeeedOled.putString("Humidity:");
  SeeedOled.setTextXY(6, 0);
  SeeedOled.putNumber(h);
  SeeedOled.setTextXY(6, 2);
  SeeedOled.putString("%");
  delay(3000);
  SeeedOled.setPageMode(); // Set addressing mode to Page Mode
  SeeedOled.clearDisplay();
  SeeedOled.setTextXY(0, 0);
  SeeedOled.putString("Visibility:");
  SeeedOled.setTextXY(1, 0);
  SeeedOled.putNumber(vis);
  SeeedOled.setTextXY(1, 2);
  SeeedOled.setTextXY(2, 0);
  SeeedOled.putString("Infrared:");
  SeeedOled.setTextXY(3, 0);
  SeeedOled.putNumber(ir);
  SeeedOled.setTextXY(4, 0);
  SeeedOled.putString("UV-Value:");
  SeeedOled.setTextXY(5, 0);
  SeeedOled.putNumber(vis);
  delay(3000);
}
